@@ -144,7 +144,7 @@ pub fn one_or_none<T: FromStr>(
     }
 }
 
-pub fn set_header_if_absent<V>(
+pub fn set_request_header_if_absent<V>(
     request: http::request::Builder,
     key: HeaderName,
     value: V,
@@ -164,6 +164,26 @@ where
     }
 }
 
+pub fn set_response_header_if_absent<V>(
+    response: http::response::Builder,
+    key: HeaderName,
+    value: V,
+) -> http::response::Builder
+where
+    HeaderValue: TryFrom<V>,
+    <HeaderValue as TryFrom<V>>::Error: Into<http::Error>,
+{
+    if !response
+        .headers_ref()
+        .map(|map| map.contains_key(&key))
+        .unwrap_or(false)
+    {
+        response.header(key, value)
+    } else {
+        response
+    }
+}
+
 /// Functions for parsing multiple comma-delimited header values out of a
 /// single header. This parsing adheres to
 /// [RFC-7230's specification of header values](https://datatracker.ietf.org/doc/html/rfc7230#section-3.2.6).
@@ -284,16 +304,17 @@ mod test {
 
     use crate::header::{
         headers_for_prefix, many_dates, read_many_from_str, read_many_primitive,
-        set_header_if_absent, ParseError,
+        set_request_header_if_absent, set_response_header_if_absent, ParseError,
     };
 
     use super::quote_header_value;
 
     #[test]
-    fn put_if_absent() {
+    fn put_on_request_if_absent() {
         let builder = http::Request::builder().header("foo", "bar");
-        let builder = set_header_if_absent(builder, HeaderName::from_static("foo"), "baz");
-        let builder = set_header_if_absent(builder, HeaderName::from_static("other"), "value");
+        let builder = set_request_header_if_absent(builder, HeaderName::from_static("foo"), "baz");
+        let builder =
+            set_request_header_if_absent(builder, HeaderName::from_static("other"), "value");
         let req = builder.body(()).expect("valid request");
         assert_eq!(
             req.headers().get_all("foo").iter().collect::<Vec<_>>(),
@@ -305,6 +326,27 @@ mod test {
         );
     }
 
+    #[test]
+    fn put_on_response_if_absent() {
+        let builder = http::Response::builder().header("foo", "bar");
+        let builder = set_response_header_if_absent(builder, HeaderName::from_static("foo"), "baz");
+        let builder =
+            set_response_header_if_absent(builder, HeaderName::from_static("other"), "value");
+        let response = builder.body(()).expect("valid response");
+        assert_eq!(
+            response.headers().get_all("foo").iter().collect::<Vec<_>>(),
+            vec!["bar"]
+        );
+        assert_eq!(
+            response
+                .headers()
+                .get_all("other")
+                .iter()
+                .collect::<Vec<_>>(),
+            vec!["value"]
+        );
+    }
+
     #[test]
     fn parse_floats() {
         let test_request = http::Request::builder()
